% title Stylized fact checking
% author(s) Hélène Arduin, Guillaume Chérel
% date 2018-05-28

# Intro

This file describes simulation experiments with source code. An html file and all source code files are generated by running the script `./make.hs`


# Requirements:

- [Stack](haskellstack.org) must be installed on your system:
  - On linux and OS X systems, do `curl -sSL https://get.haskellstack.org/ | sh`

# Introduction

Différents pathogènes peuvent interagir entre eux. Dans le cas particulier de la grippe et du pneumocoque que l'on propose d'étudier ici, être porteur de la première peut accroître les risques d'être colonisé par le second, et être porteur des deux à la fois accroître le risque de le transmettre à un autre ou de développer une infection à pneumocoque (qui n'est pas nécessaire en cas de colonisation).

Pour détecter la présence d'une interaction entre deux pathogènes, une approche consiste à utiliser un modèle dynamique qui représente le nombre d'individus sains, contagieux, infectés et les taux de transfert entre ces compartiments.

Le modèle est capable par construction de représenter dans ses paramètres les interactions entre pathogènes (formules a expliciter ci-dessous). On veut tester que lorsqu'on ajuste le modèle à des données qui ont été générées par un processus contenant de l'interaction, les paramètres résultants reflètent bien cette interaction.


# Formalisation de la démarche

L'objectif de la démarche présentée dans ce document est de construire un détecteur d'interaction entre pathogènes à partir de données. On essaie dans cette section de la formaliser.

Soit $c$ un processus de collecte de données. Ce peut être un modèle multi-agent ou une récolte de données de terrain. On représente ce processus par la fonction $c: P \rightarrow Y$, qui à un processus de dynamique d'évolution de la grippe et du pneumocoque associe les séries temporelles du nombre d'infectés par chacun et de colonisés par le pneumocoque. Un élément de P pourrait être un modèle multi-agent, et la fonction $c$ représenterait alors la simulation du modèle, ou il pourrait s'agir des processus naturels physiques, biologiques et sociaux à l'œuvre dans la société et $c$ représenterait la collecte des données de terrain. Soit $s$ la fonction qui représente le modèle SCI présenté ici, $s(\beta_0, \xi, p_0, \pi) \in P$. On peut obtenir la valeur de $c(s(\beta_0, \xi, p_0, \pi))$ par simulation, en utilisant un solveur d'équation différentielles avec celles qui définissent le modèle.

Soit $i: P \rightarrow I$ une fonction qui donne le type d'interaction à l'œuvre dans un processus d'évolution de pathogènes, $I = \{<différents types d'interaction>\}$. On sait exprimer la fonction $i$ pour $s(\beta_0, \xi, p_0, \pi)$: $i(s(\beta_0, \xi, p_0, \pi))$ vaut <détailler l'intéraction en fonction des valeurs des arguments de s>.

Rappelons que l'on veut construire un détecteur d'interactions entre pathogènes. On voudrait montrer que la procédure qui consiste à étalonner le modèle $s$ avec des données $c(g)$ issues d'un processus dynamique d'évolution des pathogènes $g \in P$ donne un détecteur d'interaction entre pathogènes dans le processus $g$. C'est-à-dire que s'il existe une interaction entre pathogènes dans $g$, alors on retrouve la même interaction dans $s(\beta_0, \xi, p_0, \pi)$ et inversement, où les paramètres $(\beta_0, \xi, p_0, \pi)$ sont fixés aux valeurs trouvées par l'étalonnage. Ceci revient à:

$\forall g \in M, i(g) = i(\text{calib}(m,c(g)))$

On voit immédiatement que cette égalité est impossible à établir quel que soit g, parce qu'on ne peut pas exprimer la fonction $i$ pour tous les processus $g \in P$. En particulier, on ne peut pas l'exprimer pour le cas qui nous interesse le plus, c'est-à-dire quand $g$ se réfère à l'évolution des pathogènes parmis les êtres humains. Si on le pouvait, il serait inutile de chercher à construire un détecteur dont c'est précisement l'objectif.

On peut toutefois trouver d'autres processus pour lesquels on peut la tester. Soit $a(\theta) \in P$ le modèle multi-agent qui prend les paramètres $\theta$ et représente l'évolution des pathogènes dans une société humaine (Ref). Les paramètres $\theta$ contrôlent explicitement le type d'interaction à l'œuvre dans la dynamique. On peut donc exprimer la fonction $i$ pour $a(\theta)$ (détailler). Et on peut alors tester l'égalité précédente avec un tel modèle.

On teste cette procédure dans un cadre dans lequel on peut la tester, et on espère que si elle fonctionne pour certains processus, elle fonctionne aussi pour les autres. On ne peut pas en avoir la certitude. Mais en multipliant les processus avec lesquels on la teste, on peut tenter d'établir des conditions sous lesquelles elle fonctionne ou non. En comparant ces conditions à la réalité, on peut argumenter en faveur ou défaveur de la validité de ce détecteur appliqué à des données de terrain, pour établir l'existence ou non d'une interaction entre pathogènes dans la réalité.


# Modélisation de l'interaction entre deux pathogènes

On représente l'interaction entre deux pathogènes avec un modèle SCI à 3 compartiments: Sensibles, Colonisés, Infectés. Le modèle est représenté comme une fonction qui prend en argument un vecteur de réels qui donnent les pas de temps auxquels on veut connaître le nombre d'individus dans chaque compartiment, un état initial de la population, et les paramètres qui décrivent les taux de transmission.

~~~~ {.R file="src/modeleLulla.R"}
##
## Modele compartimental du papier de Lulla
## 

SCI <- function(t, pop, param) {
    tt <- floor(t)
~~~~~~~~~~~~

La population initiale donne le nombre d'individus sensibles, colonisés et infectés à l'instant 0.
    
~~~~ {.R file="src/modele.R"}
    S    <- pop[1]  # Susceptibles
    C    <- pop[2]  # Colonises
    Icum <- pop[3]  # Infectes
~~~~~~~~~~~~

Les paramètres du modèle sont le taux de transmission intrinsèque du pneumocoque $BETA_0$, l'impact de l'infection par la grippe sur la transmission du pneumocoque $XI$, le taux d'infection intrinsèque du pneumocoque $P_0$, l'impact de l'infection par la grippe sur la survenue d'une infection à pneumocoque $PI$, le taux de décolonisation $L$ et la taille de la population $N$.
    
~~~~ {.R file="src/modele.R"}
    BETA_0 <- param[[1]] # transmission rate of Sp
    XI  <- param[[2]]    # multiplicative factor modifying BETA_0
    P_0 <- param[[3]]    # pathogenicity of Sp
    PI <- param[[4]]     # multiplicative factor modifying P_0
    L  <- param[[5]]     # 1/L : duration of colonisation
    N  <- param[[6]]     # population size
~~~~~~~~~~~~

Le modèle prend aussi en paramètre une série temporelle qui décrit l'évolution du nombre d'individus infectés par la grippe au cours du temps.
    
~~~~ {.R file="src/modele.R"}
    data_flu <- param[[7]]
~~~~~~~~~~~~

L'évolution du nombre d'individus dans chaque compartiment est décrite par des équations différentielles. <Schéma + équations>

~~~~ {.R file="src/modele.R"}
    BETA <- BETA_0*(1 + XI*data_flu[tt])
    P <- P_0*(1 + PI*data_flu[tt])
    
    dS <- L*C - BETA*C*S/N
    dC <- -L*C + BETA*C*S/N
    dIcum <- P*C
~~~~~~~~~~~~

La sortie du modèle est une liste des nombres d'individus dans chaque compartiment à chaque pas de temps.
    
~~~~ {.R file="src/modele.R"}
    res <- c(dS, dC, dIcum)
    list(res)
}
~~~~~~~~~~~~


# Simulation: génération de séries temporelles à partir du modèle

Le script suivant gère l'exécution du modèle pour un jeu de paramètres donné, et présente la liste de sortie du modèle comme un tableau de données.

~~~~ {.R file="src/simulation.R"}
# Execution avec un jeu de paramètres donne

simulation <- function(estim_par, fixed_par, data_flu, condInit, temps, modele)
{
    param = list(BETA_0 = estim_par[1], XI = estim_par[2], P_0 = estim_par[3], P = estim_par[4],
                 L = fixed_par[1], N = fixed_par[2],
                 data_flu = data_flu)
    
    # Execute
    result <- lsoda(y = condInit, times = temps, func = modele, parms = param)
    tab <- as.data.frame(result)
    names(tab) = c("time", "S", "C", "dIcum")
    
    tab$I[1] = tab$dIcum[1]
    for (i in 2:length(tab$dIcum))
    {
        tab$I[i] <- tab$dIcum[i] - tab$dIcum[i-1]
    }
    
    # list(tab$S, tab$C, tab$Inc)
    return(tab)
}
~~~~~~~~~~~~

Les fonctions précédentes sont regroupées au sein du script de simulation suivant.

(Chargement de bibliothèques nécessaires)

~~~~ {.R file="src/main.R"}
library(deSolve)
library(dplyr)
~~~~~~~~~~~~

La simulation du modèle a besoin de séries temporelles de l'évolution de la grippe. On utilise les données générées par le modèle agent SimFI <référence>. Plusieurs jeux de données sont générés avec différents types d'interaction entre les pathogènes. Pour exécuter une simulation, on charge le jeu de données qui correspond au scénario (type d'interaction) souhaité grâce au script OpenMOLE qui sera présenté plus bas (la variable "scenario" n'est donc pas encore définie pour l'instant).
< préparer un ou plusieurs datasets de 10 ans de données >

~~~~ {.R file="src/main.R"}
load(paste("data", scenario, sep = "/"))
prior <- dataset
~~~~~~~~~~~~

(Chargement de dépendances.)

~~~~ {.R file="src/main.R"}
if (nrow(prior))
{
    source("src/modele.R")
    source("src/simulation.R")
~~~~~~~~~~~~

On échantillonne temporellement l'état du modèle (le nombre d'individus dans chaque compartiment) avec un pas de temps de $1$, pour $T_{max}$ pas de temps.
    
~~~~ {.R file="src/main.R"}
    dt = 1
    Tmax = nrow(prior)
    Time = seq(1, Tmax, dt)
~~~~~~~~~~~~
    
Le taux de décolonisation $\lambda$ est fixé à 0.33 et la taille totale de la population $N$ est fixée à $10^5$. Les paramètres de transmission $\beta_0$ et $p_0$ ainsi que les paramètres d'interaction $\xi$ et $\pi$ sont laissés libres : ce sont eux que l'on cherche à ajuster par la procédure d'étalonnage, comme expliqué plus bas. Ces variables seront déclarées dans le script OpenMOLE plus tard.
    
~~~~ {.R file="src/main.R"}
    p0 <- 10^(-p0)
    estim <- c(beta0, xi, p0, Pi)
    lambda <- 0.33
    N <- 1e5
    fixed <- c(lambda, N)
~~~~~~~~~~~~

La proportion initiale d'individus colonisés $C0$ est fixée à 20%. La proportion d'individus sains correspond au reste de la population $(1 - C0/100) * N$.

~~~~ {.R file="src/main.R"}
    C0 <- 20
    S0 = (1 - C0/100)*N
    Init.cond = c(S0, C0*N/100, 0)
    flu <- prior$flu/N
~~~~~~~~~~~~

En exécutant une simulation du modèle, on récupère le tableau de données d'évolution du nombre d'individus dans chaque compartiment.

~~~~ {.R file="src/main.R"}
    
    simu <- simulation(estim, fixed, flu, Init.cond, Time, SCI)
~~~~~~~~~~~~


# Calcul de la vraisemblance du modèle sachant les séries temporelles de grippe données.

Après avoir exécuté le modèle en simulation, on peut calculer la vraisemblance de sa sortie sachant les séries temporelles de grippe données pour savoir à quel point les valeurs des paramètres $\beta_0$, $\xi$, $p_0$, $\pi$ permettent de reproduire ces données.

On calcule la log-vraisemblance avec une loi de Poisson.

~~~~ {.R file="src/logLik_poisson.R"}
logLik_poisson <- function(simul, data)
{
    LL <- 0
    
    # rajouter condition si trop de 0
    if (length(simul) < length(data) | length(which(simul < 0.01 | is.na(simul) | is.nan(simul))) > 200)
    {return(NA)}
    
    for (t in 1:length(data))
    {
        if (!is.nan(log(simul[t])) & !is.na(log(simul[t])) & simul[t] != 0)
        {
            LL <- LL + data[t]*log(simul[t]) - simul[t]
        }
    }
    return(LL)
}

logLik_0 <- function(simul, data)
{
    return(data[1]*log(simul[1]) - simul[1])
}


logLik_poisson_portage <- function(simul_col, N)
{
    LL <- 0
    portage_moyen <- mean(simul_col)
    LL <- LL + 0.3*N*log(portage_moyen) - portage_moyen
    return(LL)
}
~~~~~~~~~~~~

On récupère la valeur de vraisemblance dans le script principal.

~~~~ {.R file="src/main.R"}
    source("src/logLik_poisson.R")
    LL <- logLik_poisson(simu$Inc, prior$infected) + logLik_poisson_portage(simu$C, Init.cond[1] + Init.cond[2])
    
    print(LL)
    
    if (is.na(LL))
    { loglikOM <- 1e9 } else
    { loglikOM <- -LL }
} else
{ loglikOM <- 1e9 }
~~~~~~~~~~~~


# Étalonnage.

Le but de l'étalonnage est de trouver des valeurs de $\beta_0$, $\xi$, $p_0$, $\pi$ qui maximisent la vraisemblance du modèle par rapport à des séries temporelles de grippe données.
<Déclarer l'environnement directement dans le script calib.oms>

~~~~ {.scala file="src/calib.oms"}
import _parent_._parent_.envDef._
~~~~~~~~~~~~

L'étalonnage est fait avec OpenMOLE. Il faut d'abord dire à OpenMOLE que l'on va utiliser un certain nombre de variables: les variables d'entrées ($\beta_0$, $\xi$, $p_0$, $\pi$), une variable qui décrit le scénario de données artificielles à utiliser et une variable pour la log-vraisemblance de la sortie du modèle. 

~~~~ {.scala file="src/calib.oms"}
// Variables
val beta = Val[Double]
val xi = Val[Double]
val p0 = Val[Double]
val Pi = Val[Double]

// Données
val scenario = Val[String]

// Outputs
val loglikOM = Val[Double]
~~~~~~~~~~~~

On lui indique aussi comment exécuter une simulation du modèle à partir du code R décrit plus haut.

~~~~ {.scala file="src/calib.oms"}
// R task
def rTask(fichier: File) =
  RTask("""
    source("main.R")
    """,
    //install = Seq("apt update", "apt install -y gdal-bin"),
    libraries = Seq("deSolve", "dplyr")
  ) set(
    resources += workDirectory / "main.R",
    resources += workDirectory / "data",
    resources += workDirectory / "RFiles",
    rInputs += (beta, xi, p0, Pi, scenario),
    rOutputs += (loglikOM),
      
    scenario := s"noSeason/${fichier.getName}"
  )
~~~~~~~~~~~~

Différents scenarii ont été générés avec différents types d'interactions entre les pathogènes. On va faire un étalonnage pour chacun, pour tester que la procédure d'étalonnage du modèle est capable de détecter tous les types d'interactions correspondants.

~~~~ {.scala file="src/calib.oms"}
val scenarii =
  for {
    folder <- (workDirectory / "data").listFiles
    file <- folder.listFiles
  } yield s"${folder.getName}/${file.getName}"
~~~~~~~~~~~~

<Rapide explication de la méthode d'étalonnage par optimisation> On lance l'étalonnage des paramètres $\beta_0$, $\xi$, $p_0$, $\pi$ du modèle pour chaque scenario, en autorisant ces paramètres à varier respectivement dans $[0,5]$, $[-0.01, 150]$, $[0,5]$ et $[-0.01, 650]$.

~~~~ {.scala file="src/calib.oms"}
val nsga2 = NichedNSGA2(
  niche = Seq(scenario),
  muByNiche = 20,
  genome = Seq(
    beta in (0.0, 5.0),
    xi in (-0.001, 150.0),
    p0 in (0.0, 5.0),
    Pi in (-0.001, 650.0),
    scenario in scenarii.toVector
  ),
  objectives = Seq(loglikOM)
)
~~~~~~~~~~~~

On laisse l'étalonnage s'exécuter pendant au plus 10 minutes, sur 100 jobs en parallèle.

~~~~ {.scala file="src/calib.oms"}
val evolution = SteadyStateEvolution(
  algorithm = nsga2,
  evaluation = rTask,
  termination = 10 minutes
)

val island = IslandEvolution(
  evolution,
  parallelism = 100,
  termination = 5000
)
~~~~~~~~~~~~



~~~~ {.scala file="src/calib.oms"}
val savePopulationHook = SavePopulationHook(
  island,
  //evolution,
  // workDirectory / "noSeason" / s"${file.getName.dropRight(6)}"
  workDirectory / "results"
)

(island hook savePopulationHook on egi)
~~~~~~~~~~~~
